# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2014 E-MIPS (http://www.e-mips.com.ar)
#    Copyright (c) 2014 Eynes (http://www.e-mips.com.ar)
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General
#    Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from lxml import etree
from datetime import datetime
from dateutil.relativedelta import relativedelta
from dateutil.rrule import DAILY, rrule, MO, TU, WE, TH, FR
from osv import osv, fields
from tools.translate import _
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT
import time


# Service Order Action Type
class service_action_type(osv.osv):
    _name = "service.action.type"
    _description = "Service Order Action Type"

    _columns = {
        'name': fields.char('Name', size=64, required=True),
        'mail_notification': fields.boolean('Mail Notification'),
        'id_tipoaccion': fields.integer("Id CM")
    }

    _defaults = {
        'mail_notification': False,
    }


# Service Order Action
class service_action(osv.osv):
    _name = "service.action"
    _description = "Service Order Actions"
    _order = "create_date desc"

    def _get_actions_changed_type(self, cr, uid, ids, context=None):
        result = {}
        for order in self.pool.get('service.order').browse(
                cr, uid, ids, context=context):
            for action in order.action_ids:
                result[action.id] = True
        return result.keys()

    _columns = {
        'name': fields.text('Description', required=False),
        'type': fields.many2one('service.action.type', 'Action Type',
                                required=True,
                                states={'done': [('readonly', True)]}),
        'order_id': fields.many2one('service.order', 'Order'),
        'create_uid': fields.many2one('res.users', 'Requested by',
                                      readonly=True),
        'user_id': fields.many2one('res.users', 'User', required=True,
                                   states={'done': [('readonly', True)]}),
        'group_id': fields.many2one('mail.group', 'Mail Group',
                                    states={'done': [('readonly', True)]}),
        'create_date': fields.datetime('Creation date', readonly=True),
        'done_date': fields.datetime('Done date', readonly=True),
        'state': fields.selection(
            [('draft', 'To Do'),
             ('done', 'Done'),
             ('cancel', 'Cancelled')], 'State', readonly=True, select=True),
        'partner_id': fields.related('order_id', 'partner_id',
                                     string="Partner", type="many2one",
                                     relation="res.partner", store=True),
        'order_type': fields.related(
            'order_id', 'type', string="Order type",
            type="many2one", relation="service.order.type",
            store={
                'service.order': (_get_actions_changed_type,
                                  ['type', 'action_ids'], 10),
            }),
    }

    _defaults = {
        'state': 'draft',
    }

    def fields_view_get(self, cr, uid, view_id=None, view_type=False,
                        context=None, toolbar=False, submenu=False):

        res = super(service_action, self).fields_view_get(
            cr, uid, view_id=view_id, view_type=view_type,
            context=context, toolbar=toolbar, submenu=submenu)

        # Modificamos la vista de search on fly,
        # para meter filtro por 48hs o 72hs habiles
        if view_type == 'search':
            today = datetime.strptime(
                fields.date.context_today(self, cr, uid, context),
                DEFAULT_SERVER_DATE_FORMAT)

            # Nos vamos 7 dias para atras
            date_ago = today-relativedelta(days=7)
            r = rrule(dtstart=date_ago, until=today, freq=DAILY, count=7,
                      byweekday=(MO, TU, WE, TH, FR))
            f72hs = r[-4].strftime(DEFAULT_SERVER_DATE_FORMAT)
            f48hs = r[-3].strftime(DEFAULT_SERVER_DATE_FORMAT)

            doc = etree.XML(res['arch'])
            for node in doc.xpath("//filter[@name='assigned_to_me']"):
                filter48hs = etree.Element("filter")
                filter48hs.set('string', '48hs')
                filter48hs.set('domain',
                               "[('create_date', '<=', '%s')])]" % (f48hs))
                filter72hs = etree.Element("filter")
                filter72hs.set('string', '72hs')
                filter72hs.set('domain',
                               "[('create_date', '<=', '%s')])]" % (f72hs))
                node.addnext(etree.Element("separator"))
                node.addnext(filter48hs)
                node.addnext(etree.Element("separator"))
                node.addnext(filter72hs)
                node.addnext(etree.Element("separator"))

            res['arch'] = etree.tostring(doc)

        return res

    def onchange_user(self, cr, uid, ids, user_id, context=None):
        user_obj = self.pool.get('res.users')
        res = {}
        if user_id:
            user = user_obj.browse(cr, uid, user_id, context=context)
            mail_groups = user.group_ids
            if mail_groups:
                res['value'] = {
                    'group_id': mail_groups[0].id,
                }
            else:
                res['value'] = {
                    'group_id': False
                }
        return res

    def mark_done(self, cr, uid, ids, context=None):
        date = time.strftime(DEFAULT_SERVER_DATE_FORMAT)
        self.write(cr, uid, ids,
                   {'state': 'done', 'done_date': date},
                   context=context)

        action = self.browse(cr, uid, ids, context=context)

        if not (uid == action.user_id.id):
            raise osv.except_osv(
                _("Error in action"),
                _("You cannot mark this action as done because you " +
                  "are not the assgined user of this action."))

        return True

    def create(self, cr, uid, vals, context=None):
        action_type_obj = self.pool.get('service.action.type')
        action_id = super(service_action, self).create(cr, uid, vals,
                                                       context=context)
        # Notificamos al usuario o al grupo
        type = vals['type']
        user_id = 'user_id' in vals and vals['user_id'] or False
        group_id = 'group_id' in vals and vals['group_id'] or False
        order_id = 'order_id' in vals and vals['order_id'] or False

        mail = action_type_obj.read(cr, uid, type, ['mail_notification'],
                                    context=context)['mail_notification']
        if mail:
            self._action_notify(cr, uid, action_id, order_id,
                                user_id, group_id, vals['name'],
                                context=context)

        return action_id

    def _get_subtype_ids(self, cr, uid, subtypes):
        subtype_ids = []
        for subtype in subtypes:
            if '.' not in subtype:
                subtype = 'mail.%s' % subtype
            ref = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, *subtype.split('.'))
            subtype_id = ref and ref[1] or False
            if subtype_id:
                subtype_ids.append(subtype_id)

        return subtype_ids

    def _action_notify(self, cr, uid, action_id, order_id,
                       user_id, group_id, desc, context=None):
        user_obj = self.pool.get('res.users')
        group_obj = self.pool.get('mail.group')
        order_obj = self.pool.get('service.order')

        if user_id:
            reads = user_obj.read(cr, uid, user_id,
                                  ['name', 'partner_id'], context=context)
            user_name = reads['name']
            partner_ids = [reads['partner_id'][0]]

            body = _("New action assigned to %s: %s") % (user_name, desc)
            user = self.pool.get('res.users').browse(cr, uid, uid,
                                                     context=context)
            author_id = user.partner_id.id
            order_obj.message_post(
                cr, uid, [order_id], body=body, subject="Action to do",
                type="notification", subtype="mt_comment", context=context,
                author_id=author_id, partner_ids=partner_ids)

        if group_id:
            reads = group_obj.read(cr, uid, group_id, ['name'],
                                   context=context)
            group_name = reads['name']
            group_obj.message_post(
                cr, uid, [group_id],
                body=_("New action assigned to %s: %s") % (group_name, desc),
                subtype='mail.mt_comment', context=context)

        return True


class res_users(osv.osv):
    _name = "res.users"
    _inherit = "res.users"

    _columns = {
        'group_ids': fields.many2many('mail.group', 'res_users_groups',
                                      'user_id', 'group_id', 'Mail Group')
    }


class mail_group(osv.osv):
    _name = "mail.group"
    _inherit = "mail.group"

    _columns = {
        'user_ids': fields.many2many('res.users', 'res_users_groups',
                                     'group_id', 'user_id', 'Users')
    }
